@@ -1,8 +1,11 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
+#
 # Copyright (C) 2016 Rahul Raturi
-# Copyright (C) 2018 Laurent Monin
+# Copyright (C) 2018 Antonio Larrosa
+# Copyright (C) 2018-2021 Laurent Monin
+# Copyright (C) 2018-2022 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,9 +21,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+
 from PyQt5 import QtCore
 
-from picard import config
+from picard.config import Option
 from picard.const import QUERY_LIMIT
 from picard.file import File
 from picard.mbjson import (
@@ -32,7 +36,7 @@ from picard.mbjson import (
 from picard.metadata import Metadata
 from picard.track import Track
 from picard.util import sort_by_similarity
-from picard.webservice.api_helpers import escape_lucene_query
+from picard.webservice.api_helpers import build_lucene_query
 
 from picard.ui.searchdialog import (
     Retry,
@@ -45,14 +49,15 @@ class TrackSearchDialog(SearchDialog):
     dialog_header_state = "tracksearchdialog_header_state"
 
     options = [
-        config.Option("persist", dialog_header_state, QtCore.QByteArray())
+        Option("persist", dialog_header_state, QtCore.QByteArray())
     ]
 
-    def __init__(self, parent):
+    def __init__(self, parent, force_advanced_search=None):
         super().__init__(
             parent,
             accept_button_title=_("Load into Picard"),
-            search_type="track")
+            search_type="track",
+            force_advanced_search=force_advanced_search)
         self.file_ = None
         self.setWindowTitle(_("Track Search Results"))
         self.columns = [
@@ -74,12 +79,12 @@ class TrackSearchDialog(SearchDialog):
         self.tagger.mb_api.find_tracks(self.handle_reply,
                                        query=text,
                                        search=True,
+                                       advanced_search=self.use_advanced_search,
                                        limit=QUERY_LIMIT)
 
-    def load_similar_tracks(self, file_):
+    def show_similar_tracks(self, file_):
         """Perform search using existing metadata information
         from the file as query."""
-        self.retry_params = Retry(self.load_similar_tracks, file_)
         self.file_ = file_
         metadata = file_.orig_metadata
         query = {
@@ -92,21 +97,13 @@ class TrackSearchDialog(SearchDialog):
             'isrc': metadata['isrc'],
         }
 
-        # Generate query to be displayed to the user (in search box).
-        # If advanced query syntax setting is enabled by user, display query in
-        # advanced syntax style. Otherwise display only track title.
-        if config.setting["use_adv_search_syntax"]:
-            query_str = ' '.join(['%s:(%s)' % (item, escape_lucene_query(value))
-                                  for item, value in query.items() if value])
+        # If advanced query syntax setting is enabled by user, query in
+        # advanced syntax style. Otherwise query only track title.
+        if self.use_advanced_search:
+            query_str = build_lucene_query(query)
         else:
             query_str = query["track"]
-
-        query["limit"] = QUERY_LIMIT
-        self.search_box_text(query_str)
-        self.show_progress()
-        self.tagger.mb_api.find_tracks(
-            self.handle_reply,
-            **query)
+        self.search(query_str)
 
     def retry(self):
         self.retry_params.function(self.retry_params.query)
@@ -124,12 +121,11 @@ class TrackSearchDialog(SearchDialog):
 
         if self.file_:
             metadata = self.file_.orig_metadata
-
-            def candidates():
-                for track in tracks:
-                    yield metadata.compare_to_track(track, File.comparison_weights)
-
-            tracks = [result.track for result in sort_by_similarity(candidates)]
+            candidates = (
+                metadata.compare_to_track(track, File.comparison_weights)
+                for track in tracks
+            )
+            tracks = (result.track for result in sort_by_similarity(candidates))
 
         del self.search_results[:]  # Clear existing data
         self.parse_tracks(tracks)
@@ -194,7 +190,7 @@ class TrackSearchDialog(SearchDialog):
                 if isinstance(self.file_.parent, Track):
                     album = self.file_.parent.album
                     self.tagger.move_file_to_track(self.file_, track["musicbrainz_albumid"], track["musicbrainz_recordingid"])
-                    if album._files == 0:
+                    if album.get_num_total_files() == 0:
                         # Remove album if it has no more files associated
                         self.tagger.remove_album(album)
                 else:
@@ -206,7 +202,7 @@ class TrackSearchDialog(SearchDialog):
             if self.file_ and getattr(self.file_.parent, 'album', None):
                 album = self.file_.parent.album
                 self.tagger.move_file_to_nat(self.file_, track["musicbrainz_recordingid"], node)
-                if album._files == 0:
+                if album.get_num_total_files() == 0:
                     self.tagger.remove_album(album)
             else:
                 self.tagger.load_nat(track["musicbrainz_recordingid"], node)
